#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "pindefs.h"
#include "makrodefs.h"
#include "servodefs.h"

/* Timeout for a Pulse from the remote conrol receiver to show up	*
 * time [s] = timeout * 10ms										*/
#define		TIMEOUT_RCI		100

/* ABSOLETE allowed minimum and maximum servo position in terms		*
 * of sv_pos														*
 * 46 = 602 us    -    180 = 2,39 ms								*/
#define		SV_POS_MIN		46
#define		SV_POS_MAX		186

extern volatile unsigned char sv_pos[3+1];
extern volatile unsigned char sv_end_pos[3*2];
extern volatile char sv_pos_gear;
extern volatile unsigned char timeout;



/*******************************************************************/
/**
 *void softuart(unsigned char data)
 *{
 *	cli();
 *	PORTB &= ~(1<<PB5);
 *	_delay_us(52);
 *	for (unsigned char i=0;i<8;i++)
 *	{
 *		if(data&1)	PORTB |=  (1<<PB5);
 *		else 		PORTB &= ~(1<<PB5);
 *		data = data>>1;
 *		_delay_us(52);
 *	}
 *	PORTB |= (1<<PB5);
 *	_delay_us(52);
 *	sei();
 *}
 **/
/*******************************************************************/



void led_delay(unsigned char ms)
{	/* simple delay for visualiszing the gear to lid movement delay */
	for(unsigned char i=0; i<ms; i++)
	{
		_delay_ms(10);
		if(!(PIN_SAV&(1<<P_SAV)))		return;
	}
}


unsigned char wait_check(unsigned char delay)
{	/* simple delay function that includes a check wether the gear	*
	 * position needs to be updated and therefore breaks out of the	*
	 * delay loop. nice, ha?										*/
	unsigned char sv_pos_gear_last = sv_pos_gear;
	if(delay<255-30)	delay += 30;
	while(delay)
	{
		delay--;
		_delay_ms(10);
		if(sv_pos_gear_last != sv_pos_gear)
			return 1;
	}
	return 0;
}


void eep_wr(unsigned char adr, unsigned char dat)
{	/* really? u expect me to comment that? ^^ */
	while(EECR & (1<<EEPE))	;
	/* Set Programming mode */
	EECR = (0<<EEPM1)|(0<<EEPM0);
	EEAR = adr;
	EEDR = dat;
	/* set EEMPE */
	EECR |= (1<<EEMPE);
	/* start eeprom write */
	EECR |= (1<<EEPE);
}
unsigned char eep_rd(unsigned char adr)
{	/* really? u expect me to comment that? ^^ */
	while(EECR & (1<<EEPE))	;
	EEAR = adr;
	/* start eeprom read */
	EECR |= (1<<EERE);
	return EEDR;
}


unsigned char adc_read(void)
{	/* return ADC value */
	/* start single conversion */
	ADCSRA |= (1<<ADSC);
	/* wait for conversion to finish */
	while(!(ADCSRA & (1<<ADIF)))	;
	return ADCH;
}



void endpos_set_mode(void)
{	/* end-posistion adjustment mode								*
	 * set the endposition of a servo according to poti-position	*
	 * read via ADC.												*
	 * if requested by button-press, save the setting to the eeprom	*/
	/* note: the actual servo is (act_servo-1)/2, but since we need	*
	 * two end-positions to be known per servo, this is the way i	*
	 * did it. sue me.												*/
	unsigned char act_swtch, act_swtch_last=0xff, act_adc, wait_for_0=1;

	/* simplest debouce */
	_delay_ms(100);
	while(!(PIN_SET&(1<<P_SET)))
	{	/* read ADC, set the servo selected by the hex-ecoding-		*
		 * switch according to the poti position within some range	*
		 * and with some offset for limiting.						*/
		/* reset the timeout counter so that no remote controll is	*
		 * needed to set the end-posisions							*
		 * re-enable the timer if needed							*/
		timeout_res();
		/* wait for servo selection switch value to be 0. this		*
		 * is a prerequesit for not reaching servo positions harm-	*
		 * to the plane												*/
		while((((~PIN_SVS)>>P_SVS0) & (0x0f)) && wait_for_0)	
			if(PIN_SET&(1<<P_SET))	return;
		/* get the servo selection switch value */
		act_swtch = (((~PIN_SVS)>>P_SVS0) & (0x0f));
		wait_for_0=0;

		/* get the poti position */
		act_adc = adc_read();

		/* after changing the servo selection switch, wait 2s for	*
		 * new position to get active								*/
		if(act_swtch != act_swtch_last)
		{
			led_rt_an();
			_delay_ms(1500);
			act_swtch_last = act_swtch;
			led_rt_aus();
			continue;
		}
		
		/* position 0 - do nothing */
		if(!act_swtch)		{_delay_ms(150); continue;}

		if(act_swtch<3*2+1)
		{	/* adjust each position for the 3 servos */
			if(act_swtch%2)
			{	/* check for left (else: right) end position	*/
				/* set current servo position according to poti */
				sv_pos[(act_swtch-1)/2] = SV_POS_MAX - (act_adc/3);
			}
			else
			{
				sv_pos[(act_swtch-1)/2] = SV_POS_MIN + (act_adc/3);
			}
		}
		else if(act_swtch==7)
		{	/* open the lid */
			sv_pos[BUG_KLAPPE]  = sv_end_pos[BUG_KLAPPE_AUF];
		}
		else if(act_swtch==8)
		{	/* close the lid */
			sv_pos[BUG_KLAPPE]  = sv_end_pos[BUG_KLAPPE_ZU];
		}
		else if(act_swtch==9)
		{	/* open the lid */
			sv_pos[HECK_KLAPPE] = sv_end_pos[HECK_KLAPPE_AUF];
		}
		else if(act_swtch==10)
		{	/* close the lid */
			sv_pos[HECK_KLAPPE] = sv_end_pos[HECK_KLAPPE_ZU];
		}
		else if(act_swtch==11)
		{	/* bring gear out */
			sv_pos[FAHRW] = sv_end_pos[FAHRW_AUF];
		}
		else if(act_swtch==12)
		{	/* bring gear in */
			sv_pos[FAHRW] = sv_end_pos[FAHRW_ZU];
		}
		else if(act_swtch>14)	{_delay_ms(150); continue;}
		else if(act_swtch>=13)
		{	/* adjust delay between gear and lid movement */
			led_rt_tgl();
			if(act_adc<255-30)	act_adc += 30;
			led_delay(act_adc-15);
		}

		if(!(PIN_SAV&(1<<P_SAV)))
		{	/* check if save button has been pressed.	*
			 * if so, save to the eeprom and exit		*/
			/* simplest debouce */
			for(unsigned char i=0;i<5;i++)
			{
				led_rt_tgl();
				_delay_ms(100);
			}
			led_rt_aus();
			if(act_swtch<3*2+1)
			{	/* save a servo position */
				eep_wr(act_swtch-1, sv_pos[(act_swtch-1)/2]);
				/* update the servo position to the ram variable */
				sv_end_pos[act_swtch-1] = sv_pos[(act_swtch-1)/2];
			}
			else if(act_swtch>=13)
			{	/* save a delay time */
				eep_wr(act_swtch, adc_read());
			}
		}

		/* no hustle */
		_delay_ms(150);
		
		act_swtch_last = act_swtch;
	}
	led_rt_aus();
	/* instant timeout if no remote control signal present */
	timeout=TIMEOUT_RCI+1;
}